@@ -19,33 +19,43 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
+from __future__ import annotations
 
-import re
-import datetime
 import calendar
+import datetime
+import re
+from typing import Callable, TypedDict
+
+from typing_extensions import Unpack
 
-from .GetText import GetText
 from .CasingTypeEnum import CasingTypeEnum
 from .DescriptionTypeEnum import DescriptionTypeEnum
+from .Exception import FormatError, WrongArgumentError
 from .ExpressionParser import ExpressionParser
+from .GetText import GetText
 from .Options import Options
 from .StringBuilder import StringBuilder
-from .Exception import FormatException, WrongArgumentException
 
 
+class OptionsKwargs(TypedDict, total=False):
+    use_24hour_time_format: bool
+    locale_code: str
+    casing_type: CasingTypeEnum
+    verbose: bool
+    day_of_week_start_index_zero: bool
+    locale_location: str | None
+
 class ExpressionDescriptor:
-
-    """
-     Converts a Cron Expression into a human readable string
+    """Converts a Cron Expression into a human readable string
     """
 
-    _special_characters = ['/', '-', ',', '*']
+    _special_characters = ("/", "-", ",", "*")
 
-    _expression = ''
-    _options = None
-    _expression_parts = []
+    _expression = ""
+    _options: Options
+    _expression_parts: list[str]
 
-    def __init__(self, expression, options=None, **kwargs):
+    def __init__(self, expression: str, options: Options | None=None, **kwargs: Unpack[OptionsKwargs]) -> None:
         """Initializes a new instance of the ExpressionDescriptor
 
         Args:
@@ -62,11 +72,12 @@ class ExpressionDescriptor:
         self._expression_parts = []
 
         # if kwargs in _options, overwrite it, if not raise exception
-        for kwarg in kwargs:
+        for kwarg, value in kwargs.items():
             if hasattr(self._options, kwarg):
-                setattr(self._options, kwarg, kwargs[kwarg])
+                setattr(self._options, kwarg, value)
             else:
-                raise WrongArgumentException("Unknown {} configuration argument".format(kwarg))
+                msg = f"Unknown {kwarg} configuration argument"
+                raise WrongArgumentError(msg)
 
         # Initializes localization
         self.get_text = GetText(options.locale_code, options.locale_location)
@@ -75,10 +86,10 @@ class ExpressionDescriptor:
         parser = ExpressionParser(self._expression, self._options)
         self._expression_parts = parser.parse()
 
-    def _(self, message):
+    def translate(self, message: str) -> str:
         return self.get_text.trans.gettext(message)
 
-    def get_description(self, description_type=DescriptionTypeEnum.FULL):
+    def get_description(self, description_type: DescriptionTypeEnum = DescriptionTypeEnum.FULL) -> str:
         """Generates a humanreadable string for the Cron Expression
 
         Args:
@@ -103,7 +114,7 @@ class ExpressionDescriptor:
 
         return choices.get(description_type, self.get_seconds_description)()
 
-    def get_full_description(self):
+    def get_full_description(self) -> str:
         """Generates the FULL description
 
         Returns:
@@ -112,7 +123,6 @@ class ExpressionDescriptor:
             FormatException: if formatting fails
 
         """
-
         try:
             time_segment = self.get_time_of_day_description()
             day_of_month_desc = self.get_day_of_month_description()
@@ -120,24 +130,19 @@ class ExpressionDescriptor:
             day_of_week_desc = self.get_day_of_week_description()
             year_desc = self.get_year_description()
 
-            description = "{0}{1}{2}{3}{4}".format(
-                time_segment,
-                day_of_month_desc,
-                day_of_week_desc,
-                month_desc,
-                year_desc)
+            description = f"{time_segment}{day_of_month_desc}{day_of_week_desc}{month_desc}{year_desc}"
 
-            description = self.transform_verbosity(description, self._options.verbose)
+            description = self.transform_verbosity(description, use_verbose_format=self._options.verbose)
             description = ExpressionDescriptor.transform_case(description, self._options.casing_type)
-        except Exception:
-            description = self._(
-                "An error occurred when generating the expression description.  Check the cron expression syntax."
+        except Exception as e:
+            description = self.translate(
+                "An error occurred when generating the expression description.  Check the cron expression syntax.",
             )
-            raise FormatException(description)
+            raise FormatError(description) from e
 
         return description
 
-    def get_time_of_day_description(self):
+    def get_time_of_day_description(self) -> str:
         """Generates a description for only the TIMEOFDAY portion of the expression
 
         Returns:
@@ -155,7 +160,7 @@ class ExpressionDescriptor:
             any(exp in hour_expression for exp in self._special_characters) is False and \
                 any(exp in seconds_expression for exp in self._special_characters) is False:
             # specific time of day (i.e. 10 14)
-            description.append(self._("At "))
+            description.append(self.translate("At "))
             description.append(
                 self.format_time(
                     hour_expression,
@@ -165,14 +170,14 @@ class ExpressionDescriptor:
             "," not in minute_expression and \
                 any(exp in hour_expression for exp in self._special_characters) is False:
             # minute range in single hour (i.e. 0-10 11)
-            minute_parts = minute_expression.split('-')
-            description.append(self._("Every minute between {0} and {1}").format(
+            minute_parts = minute_expression.split("-")
+            description.append(self.translate("Every minute between {0} and {1}").format(
                 self.format_time(hour_expression, minute_parts[0]), self.format_time(hour_expression, minute_parts[1])))
         elif seconds_expression == "" and "," in hour_expression and "-" not in hour_expression and \
                 any(exp in minute_expression for exp in self._special_characters) is False:
             # hours list with single minute (o.e. 30 6,14,16)
-            hour_parts = hour_expression.split(',')
-            description.append(self._("At"))
+            hour_parts = hour_expression.split(",")
+            description.append(self.translate("At"))
             for i, hour_part in enumerate(hour_parts):
                 description.append(" ")
                 description.append(self.format_time(hour_part, minute_expression))
@@ -181,7 +186,7 @@ class ExpressionDescriptor:
                     description.append(",")
 
                 if i == len(hour_parts) - 2:
-                    description.append(self._(" and"))
+                    description.append(self.translate(" and"))
         else:
             # default time description
             seconds_description = self.get_seconds_description()
@@ -201,7 +206,7 @@ class ExpressionDescriptor:
             description.append(hours_description)
         return str(description)
 
-    def get_seconds_description(self):
+    def get_seconds_description(self) -> str:
         """Generates a description for only the SECONDS portion of the expression
 
         Returns:
@@ -209,29 +214,29 @@ class ExpressionDescriptor:
 
         """
 
-        def get_description_format(s):
+        def get_description_format(s: str) -> str:
             if s == "0":
                 return ""
 
             try:
                 if int(s) < 20:
-                    return self._("at {0} seconds past the minute")
-                else:
-                    return self._("at {0} seconds past the minute [grThen20]") or self._("at {0} seconds past the minute")
+                    return self.translate("at {0} seconds past the minute")
+
+                return self.translate("at {0} seconds past the minute [grThen20]") or self.translate("at {0} seconds past the minute")
             except ValueError:
-                return self._("at {0} seconds past the minute")
+                return self.translate("at {0} seconds past the minute")
 
         return self.get_segment_description(
             self._expression_parts[0],
-            self._("every second"),
+            self.translate("every second"),
             lambda s: s,
-            lambda s: self._("every {0} seconds").format(s),
-            lambda s: self._("seconds {0} through {1} past the minute"),
+            lambda s: self.translate("every {0} seconds").format(s),
+            lambda _: self.translate("seconds {0} through {1} past the minute"),
             get_description_format,
-            lambda s: self._(", second {0} through second {1}") or self._(", {0} through {1}")
+            lambda _: self.translate(", second {0} through second {1}") or self.translate(", {0} through {1}"),
         )
 
-    def get_minutes_description(self):
+    def get_minutes_description(self) -> str:
         """Generates a description for only the MINUTE portion of the expression
 
         Returns:
@@ -240,29 +245,29 @@ class ExpressionDescriptor:
         """
         seconds_expression = self._expression_parts[0]
 
-        def get_description_format(s):
+        def get_description_format(s: str) -> str:
             if s == "0" and seconds_expression == "":
                 return ""
 
             try:
                 if int(s) < 20:
-                    return self._("at {0} minutes past the hour")
-                else:
-                    return self._("at {0} minutes past the hour [grThen20]") or self._("at {0} minutes past the hour")
+                    return self.translate("at {0} minutes past the hour")
+
+                return self.translate("at {0} minutes past the hour [grThen20]") or self.translate("at {0} minutes past the hour")
             except ValueError:
-                return self._("at {0} minutes past the hour")
+                return self.translate("at {0} minutes past the hour")
 
         return self.get_segment_description(
             self._expression_parts[1],
-            self._("every minute"),
+            self.translate("every minute"),
             lambda s: s,
-            lambda s: self._("every {0} minutes").format(s),
-            lambda s: self._("minutes {0} through {1} past the hour"),
+            lambda s: self.translate("every {0} minutes").format(s),
+            lambda _: self.translate("minutes {0} through {1} past the hour"),
             get_description_format,
-            lambda s: self._(", minute {0} through minute {1}") or self._(", {0} through {1}")
+            lambda _: self.translate(", minute {0} through minute {1}") or self.translate(", {0} through {1}"),
         )
 
-    def get_hours_description(self):
+    def get_hours_description(self) -> str:
         """Generates a description for only the HOUR portion of the expression
 
         Returns:
@@ -272,72 +277,71 @@ class ExpressionDescriptor:
         expression = self._expression_parts[2]
         return self.get_segment_description(
             expression,
-            self._("every hour"),
+            self.translate("every hour"),
             lambda s: self.format_time(s, "0"),
-            lambda s: self._("every {0} hours").format(s),
-            lambda s: self._("between {0} and {1}"),
-            lambda s: self._("at {0}"),
-            lambda s: self._(", hour {0} through hour {1}") or self._(", {0} through {1}")
+            lambda s: self.translate("every {0} hours").format(s),
+            lambda _: self.translate("between {0} and {1}"),
+            lambda _: self.translate("at {0}"),
+            lambda _: self.translate(", hour {0} through hour {1}") or self.translate(", {0} through {1}"),
         )
 
-    def get_day_of_week_description(self):
+    def get_day_of_week_description(self) -> str:
         """Generates a description for only the DAYOFWEEK portion of the expression
 
         Returns:
             The DAYOFWEEK description
 
         """
-
         if self._expression_parts[5] == "*":
             # DOW is specified as * so we will not generate a description and defer to DOM part.
             # Otherwise, we could get a contradiction like "on day 1 of the month, every day"
             # or a dupe description like "every day, every day".
             return ""
 
-        def get_day_name(s):
+        def get_day_name(s: str) -> str:
             exp = s
             if "#" in s:
                 exp, _ = s.split("#", 2)
             elif "L" in s:
-                exp = exp.replace("L", '')
+                exp = exp.replace("L", "")
             return ExpressionDescriptor.number_to_day(int(exp))
 
-        def get_format(s):
+        def get_format(s: str) -> str:
             if "#" in s:
                 day_of_week_of_month = s[s.find("#") + 1:]
 
                 try:
                     day_of_week_of_month_number = int(day_of_week_of_month)
                     choices = {
-                        1: self._("first"),
-                        2: self._("second"),
-                        3: self._("third"),
-                        4: self._("fourth"),
-                        5: self._("fifth"),
+                        1: self.translate("first"),
+                        2: self.translate("second"),
+                        3: self.translate("third"),
+                        4: self.translate("fourth"),
+                        5: self.translate("fifth"),
                     }
-                    day_of_week_of_month_description = choices.get(day_of_week_of_month_number, '')
+                    day_of_week_of_month_description = choices.get(day_of_week_of_month_number, "")
                 except ValueError:
-                    day_of_week_of_month_description = ''
+                    day_of_week_of_month_description = ""
 
-                formatted = "{}{}{}".format(self._(", on the "), day_of_week_of_month_description, self._(" {0} of the month"))
+                formatted = "{}{}{}".format(self.translate(", on the "), day_of_week_of_month_description, self.translate(" {0} of the month"))
             elif "L" in s:
-                formatted = self._(", on the last {0} of the month")
+                formatted = self.translate(", on the last {0} of the month")
             else:
-                formatted = self._(", only on {0}")
+                formatted = self.translate(", only on {0}")
 
             return formatted
 
         return self.get_segment_description(
             self._expression_parts[5],
-            self._(", every day"),
+            self.translate(", every day"),
             lambda s: get_day_name(s),
-            lambda s: self._(", every {0} days of the week").format(s),
-            lambda s: self._(", {0} through {1}"),
+            lambda s: self.translate(", every {0} days of the week").format(s),
+            lambda _: self.translate(", {0} through {1}"),
             lambda s: get_format(s),
-            lambda s: self._(", {0} through {1}")
+            lambda _: self.translate(", {0} through {1}"),
         )
 
-    def get_month_description(self):
+    def get_month_description(self) -> str:
         """Generates a description for only the MONTH portion of the expression
 
         Returns:
@@ -346,15 +350,15 @@ class ExpressionDescriptor:
         """
         return self.get_segment_description(
             self._expression_parts[4],
-            '',
-            lambda s: datetime.date(datetime.date.today().year, int(s), 1).strftime("%B"),
-            lambda s: self._(", every {0} months").format(s),
-            lambda s: self._(", month {0} through month {1}") or self._(", {0} through {1}"),
-            lambda s: self._(", only in {0}"),
-            lambda s: self._(", month {0} through month {1}") or self._(", {0} through {1}")
+            "",
+            lambda s: datetime.date(datetime.datetime.now(tz=datetime.timezone.utc).date().year, int(s), 1).strftime("%B"),
+            lambda s: self.translate(", every {0} months").format(s),
+            lambda _: self.translate(", month {0} through month {1}") or self.translate(", {0} through {1}"),
+            lambda _: self.translate(", only in {0}"),
+            lambda _: self.translate(", month {0} through month {1}") or self.translate(", {0} through {1}"),
         )
 
-    def get_day_of_month_description(self):
+    def get_day_of_month_description(self) -> str:
         """Generates a description for only the DAYOFMONTH portion of the expression
 
         Returns:
@@ -364,38 +368,42 @@ class ExpressionDescriptor:
         expression = self._expression_parts[3]
 
         if expression == "L":
-            description = self._(", on the last day of the month")
-        elif expression == "LW" or expression == "WL":
-            description = self._(", on the last weekday of the month")
+            description = self.translate(", on the last day of the month")
+        elif expression in ("LW", "WL"):
+            description = self.translate(", on the last weekday of the month")
         else:
             regex = re.compile(r"(\d{1,2}W)|(W\d{1,2})")
             m = regex.match(expression)
             if m:  # if matches
                 day_number = int(m.group().replace("W", ""))
 
-                day_string = self._("first weekday") if day_number == 1 else self._("weekday nearest day {0}").format(day_number)
-                description = self._(", on the {0} of the month").format(day_string)
+                day_string = self.translate("first weekday") if day_number == 1 else self.translate("weekday nearest day {0}").format(day_number)
+                description = self.translate(", on the {0} of the month").format(day_string)
+            elif expression == "*" and self._expression_parts[5] != "*":
+                # DOW is specified, but DOM is *, so do not generate DOM description.
+                # Otherwise, we could get a contradiction like "every day, on Tuesday"
+                description = ""
             else:
                 # Handle "last day offset"(i.e.L - 5: "5 days before the last day of the month")
                 regex = re.compile(r"L-(\d{1,2})")
                 m = regex.match(expression)
                 if m:  # if matches
                     off_set_days = m.group(1)
-                    description = self._(", {0} days before the last day of the month").format(off_set_days)
+                    description = self.translate(", {0} days before the last day of the month").format(off_set_days)
                 else:
                     description = self.get_segment_description(
                         expression,
-                        self._(", every day"),
+                        self.translate(", every day"),
                         lambda s: s,
-                        lambda s: self._(", every day") if s == "1" else self._(", every {0} days"),
-                        lambda s: self._(", between day {0} and {1} of the month"),
-                        lambda s: self._(", on day {0} of the month"),
-                        lambda s: self._(", {0} through {1}")
+                        lambda s: self.translate(", every day") if s == "1" else self.translate(", every {0} days"),
+                        lambda _: self.translate(", between day {0} and {1} of the month"),
+                        lambda _: self.translate(", on day {0} of the month"),
+                        lambda _: self.translate(", {0} through {1}"),
                     )
 
         return description
 
-    def get_year_description(self):
+    def get_year_description(self) -> str:
         """Generates a description for only the YEAR portion of the expression
 
         Returns:
@@ -403,36 +411,36 @@ class ExpressionDescriptor:
 
         """
 
-        def format_year(s):
+        def format_year(s: str) -> str:
             regex = re.compile(r"^\d+$")
             if regex.match(s):
                 year_int = int(s)
                 if year_int < 1900:
-                    return year_int
+                    return str(year_int)
                 return datetime.date(year_int, 1, 1).strftime("%Y")
-            else:
-                return s
+
+            return s
 
         return self.get_segment_description(
             self._expression_parts[6],
-            '',
+            "",
             lambda s: format_year(s),
-            lambda s: self._(", every {0} years").format(s),
-            lambda s: self._(", year {0} through year {1}") or self._(", {0} through {1}"),
-            lambda s: self._(", only in {0}"),
-            lambda s: self._(", year {0} through year {1}") or self._(", {0} through {1}")
+            lambda s: self.translate(", every {0} years").format(s),
+            lambda _: self.translate(", year {0} through year {1}") or self.translate(", {0} through {1}"),
+            lambda _: self.translate(", only in {0}"),
+            lambda _: self.translate(", year {0} through year {1}") or self.translate(", {0} through {1}"),
         )
 
     def get_segment_description(
         self,
-        expression,
-        all_description,
-        get_single_item_description,
-        get_interval_description_format,
-        get_between_description_format,
-        get_description_format,
-        get_range_format
-    ):
+        expression: str,
+        all_description: str,
+        get_single_item_description: Callable[[str], str],
+        get_interval_description_format: Callable[[str], str],
+        get_between_description_format: Callable[[str], str],
+        get_description_format: Callable[[str], str],
+        get_range_format: Callable[[str], str],
+    ) -> str:
         """Returns segment description
         Args:
             expression: Segment to descript
@@ -446,39 +454,43 @@ class ExpressionDescriptor:
             segment description
 
         """
-        description = None
-        if expression is None or expression == '':
-            description = ''
-        elif expression == "*":
-            description = all_description
-        elif any(ext in expression for ext in ['/', '-', ',']) is False:
-            description = get_description_format(expression).format(get_single_item_description(expression))
-        elif "/" in expression:
-            segments = expression.split('/')
-            description = get_interval_description_format(segments[1]).format(get_single_item_description(segments[1]))
+        if not expression:
+            return ""
+
+        if expression == "*":
+            return all_description
+
+        if not any(ext in expression for ext in ["/", "-", ","]):
+            return get_description_format(expression).format(get_single_item_description(expression))
+
+        if "/" in expression:
+            segments = expression.split("/")
+            description = get_interval_description_format(segments[1]).format(segments[1])
 
             # interval contains 'between' piece (i.e. 2-59/3 )
             if "-" in segments[0]:
                 between_segment_description = self.generate_between_segment_description(
                     segments[0],
                     get_between_description_format,
-                    get_single_item_description
+                    get_single_item_description,
                 )
                 if not between_segment_description.startswith(", "):
                     description += ", "
 
                 description += between_segment_description
-            elif any(ext in segments[0] for ext in ['*', ',']) is False:
+            elif not any(ext in segments[0] for ext in ["*", ","]):
                 range_item_description = get_description_format(segments[0]).format(
-                    get_single_item_description(segments[0])
+                    get_single_item_description(segments[0]),
                 )
                 range_item_description = range_item_description.replace(", ", "")
 
-                description += self._(", starting {0}").format(range_item_description)
-        elif "," in expression:
-            segments = expression.split(',')
+                description += self.translate(", starting {0}").format(range_item_description)
+            return description
 
-            description_content = ''
+        if "," in expression:
+            segments = expression.split(",")
+
+            description_content = ""
             for i, segment in enumerate(segments):
                 if i > 0 and len(segments) > 2:
                     description_content += ","
@@ -487,13 +499,13 @@ class ExpressionDescriptor:
                         description_content += " "
 
                 if i > 0 and len(segments) > 1 and (i == len(segments) - 1 or len(segments) == 2):
-                    description_content += self._(" and ")
+                    description_content += self.translate(" and ")
 
                 if "-" in segment:
                     between_segment_description = self.generate_between_segment_description(
                         segment,
                         get_range_format,
-                        get_single_item_description
+                        get_single_item_description,
                     )
 
                     between_segment_description = between_segment_description.replace(", ", "")
@@ -502,31 +514,31 @@ class ExpressionDescriptor:
                 else:
                     description_content += get_single_item_description(segment)
 
-            description = get_description_format(expression).format(description_content)
-        elif "-" in expression:
-            description = self.generate_between_segment_description(
+            return get_description_format(expression).format(description_content)
+
+        if "-" in expression:
+            return self.generate_between_segment_description(
                 expression,
                 get_between_description_format,
-                get_single_item_description
+                get_single_item_description,
             )
 
-        return description
+        return "?"
 
     def generate_between_segment_description(
             self,
-            between_expression,
-            get_between_description_format,
-            get_single_item_description
-    ):
-        """
-        Generates the between segment description
+            between_expression: str,
+            get_between_description_format: Callable[[str], str],
+            get_single_item_description: Callable[[str], str],
+    ) -> str:
+        """Generates the between segment description
         :param between_expression:
         :param get_between_description_format:
         :param get_single_item_description:
         :return: The between segment description
         """
         description = ""
-        between_segments = between_expression.split('-')
+        between_segments = between_expression.split("-")
         between_segment_1_description = get_single_item_description(between_segments[0])
         between_segment_2_description = get_single_item_description(between_segments[1])
         between_segment_2_description = between_segment_2_description.replace(":00", ":59")
@@ -538,10 +550,10 @@ class ExpressionDescriptor:
 
     def format_time(
         self,
-        hour_expression,
-        minute_expression,
-        second_expression=''
-    ):
+        hour_expression: str,
+        minute_expression: str,
+        second_expression: str | None=None,
+    ) -> str:
         """Given time parts, will construct a formatted time description
         Args:
             hour_expression: Hours part
@@ -553,9 +565,9 @@ class ExpressionDescriptor:
         """
         hour = int(hour_expression)
 
-        period = ''
+        period = ""
         if self._options.use_24hour_time_format is False:
-            period = self._("PM") if (hour >= 12) else self._("AM")
+            period = self.translate("PM") if (hour >= 12) else self.translate("AM")
             if period:
                 # add preceding space
                 period = " " + period
@@ -567,13 +579,13 @@ class ExpressionDescriptor:
                 hour = 12
 
         minute = str(int(minute_expression))  # Removes leading zero if any
-        second = ''
+        second = ""
         if second_expression is not None and second_expression:
             second = "{}{}".format(":", str(int(second_expression)).zfill(2))
 
-        return "{0}:{1}{2}{3}".format(str(hour).zfill(2), minute.zfill(2), second, period)
+        return f"{str(hour).zfill(2)}:{minute.zfill(2)}{second}{period}"
 
-    def transform_verbosity(self, description, use_verbose_format):
+    def transform_verbosity(self, description: str, *, use_verbose_format: bool = False) -> str:
         """Transforms the verbosity of the expression description by stripping verbosity from original description
         Args:
             description: The description to transform
@@ -582,15 +594,15 @@ class ExpressionDescriptor:
             The transformed description with proper verbosity
 
         """
-        if use_verbose_format is False:
-            description = description.replace(self._(", every minute"), '')
-            description = description.replace(self._(", every hour"), '')
-            description = description.replace(self._(", every day"), '')
-            description = re.sub(r', ?$', '', description)
+        if not use_verbose_format:
+            description = description.replace(self.translate(", every minute"), "")
+            description = description.replace(self.translate(", every hour"), "")
+            description = description.replace(self.translate(", every day"), "")
+            description = re.sub(r", ?$", "", description)
         return description
 
     @staticmethod
-    def transform_case(description, case_type):
+    def transform_case(description: str, case_type: CasingTypeEnum) -> str:
         """Transforms the case of the expression description, based on options
         Args:
             description: The description to transform
@@ -599,9 +611,7 @@ class ExpressionDescriptor:
             The transformed description with proper casing
         """
         if case_type == CasingTypeEnum.Sentence:
-            description = "{}{}".format(
-                description[0].upper(),
-                description[1:])
+            description = f"{description[0].upper()}{description[1:]}"
         elif case_type == CasingTypeEnum.Title:
             description = description.title()
         else:
@@ -609,7 +619,7 @@ class ExpressionDescriptor:
         return description
 
     @staticmethod
-    def number_to_day(day_number):
+    def number_to_day(day_number: int) -> str:
         """Returns localized day name by its CRON number
 
         Args:
@@ -618,6 +628,7 @@ class ExpressionDescriptor:
             Day corresponding to day_number
         Raises:
             IndexError: When day_number is not found
+
         """
         try:
             return [
@@ -627,19 +638,20 @@ class ExpressionDescriptor:
                 calendar.day_name[2],
                 calendar.day_name[3],
                 calendar.day_name[4],
-                calendar.day_name[5]
+                calendar.day_name[5],
             ][day_number]
-        except IndexError:
-            raise IndexError("Day {} is out of range!".format(day_number))
+        except IndexError as e:
+            msg = f"Day {day_number} is out of range!"
+            raise IndexError(msg) from e
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.get_description()
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return self.get_description()
 
 
-def get_description(expression, options=None):
+def get_description(expression: str, options: Options | None=None) -> str:
     """Generates a human readable string for the Cron Expression
     Args:
         expression: The cron expression string
