@@ -2,15 +2,15 @@ import argparse
 
 from cron_descriptor import CasingTypeEnum, ExpressionDescriptor, Options
 
-parser = argparse.ArgumentParser(prog='cron_descriptor')
-parser.add_argument('expression')
-parser.add_argument('-c', '--casing',
+parser = argparse.ArgumentParser(prog="cron_descriptor")
+parser.add_argument("expression")
+parser.add_argument("-c", "--casing",
                     choices=[v for v in vars(CasingTypeEnum)
-                             if not v.startswith('_')],
-                    default='Sentence')
-parser.add_argument('-v', '--verbose', action='store_true')
-parser.add_argument('-W', '--one-indexed-week', action='store_true')
-parser.add_argument('-H', '--use-24-hour-time-format', action='store_true')
+                             if not v.startswith("_")],
+                    default="Sentence")
+parser.add_argument("-v", "--verbose", action="store_true")
+parser.add_argument("-W", "--one-indexed-week", action="store_true")
+parser.add_argument("-H", "--use-24-hour-time-format", action="store_true")
 
 args = parser.parse_args()
 
